package cz.agents.alite.vis3d;

import com.jme3.system.AppSettings;
import cz.agents.alite.vis3d.layer.DefaultSetting3DLayer;
import cz.agents.alite.vis3d.layer.Vis3DLayer;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

public class Vis3dManager {

    private Vis3d vis;
    private Vis3DLayer settingLayer;
    private final List<Vis3DLayer> layers = new CopyOnWriteArrayList<Vis3DLayer>();
    private static AppSettings appSettings;
    private static Vis3dManager instance = null;
    private static boolean showSettings;
    private static boolean isStarted = false;
    private static List<Vis3DLayer> layersToPostponedInit = new LinkedList<Vis3DLayer>();

    private Vis3dManager() {
    }

    public static void startVis() {
        getInstance().vis = new Vis3d(getInstance());
        Vis3d visRef = getInstance().vis;
        visRef.setShowSettings(showSettings);
        visRef.setSettings(getAppSettings());
        visRef.start();
        isStarted = true;
    }

    void initLayers(Vis3d vis) {
        for (Vis3DLayer layer : layers) {
            layer.init(vis);
        }
    }

    public static void setAppSetting(String key, Object value) {
        getAppSettings().put(key, value);
    }

    public static void setInitParam(String title, int width, int height) {
        getAppSettings().setWidth(width);
        getAppSettings().setHeight(height);
        getAppSettings().setTitle(title);
    }

    private static AppSettings getAppSettings() {
        if (appSettings == null) {
            appSettings = new AppSettings(true);
            initAppSettings();
        }
        return appSettings;
    }

    private static void initAppSettings() {
        appSettings.setVSync(true);
    }

    public static void registerLayer(Vis3DLayer layer) {
        getInstance().layers.add(layer);

        if (isStarted) {
            synchronized (layersToPostponedInit) {
                layersToPostponedInit.add(layer);
            }
        }
    }

    public static void registerSettingLayer(Vis3DLayer layer) {
        getInstance().settingLayer = layer;
    }

    public static synchronized Vis3dManager getInstance() {
        if (instance == null) {
            instance = new Vis3dManager();
        }
        return instance;
    }

    public void initSettingLayer(Vis3d vis3d) {
        if (settingLayer == null) {
            settingLayer = new DefaultSetting3DLayer();
        }
        settingLayer.init(vis3d);
    }

    public static void setShowSettings(boolean showSettings) {
        Vis3dManager.showSettings = showSettings;
    }

    public void initPostponedLayers() {
        if (isStarted && !layersToPostponedInit.isEmpty()) {
            synchronized (layersToPostponedInit) {
                for (Vis3DLayer layer : layersToPostponedInit) {
                    layer.init(getInstance().vis);
                }
                layersToPostponedInit.clear();
            }
        }
    }
}
