package tacv.universe.environment;

import cz.agents.alite.common.entity.Entity;
import cz.agents.alite.communication.DefaultCommunicator;
import cz.agents.alite.communication.channel.CommunicationChannelException;
import cz.agents.alite.communication.directory.DirectoryFacilitator;
import cz.agents.alite.communication.eventbased.EventBasedCommunicationChannel;
import cz.agents.alite.environment.Action;
import cz.agents.alite.environment.Sensor;
import cz.agents.alite.environment.eventbased.EventBasedEnvironment;
import java.io.File;
import tacv.universe.AgentScoutUniverse;
import tacv.universe.environment.storage.CarStorage;
import tacv.universe.map.UrbanMap;
import tacv.universe.world.CollisionZone;
import tacv.universe.world.Terrain;
import tacv.universe.world.WorldInfo;

public class AgentScoutEnvironment extends EventBasedEnvironment {

    public static final double ANGLE_TOLERANCE = 0.01;
    public static final double DISTANCE_TOLERANCE = 0.01;
    private AgentScoutUniverse universe;
    private AgentScoutEnvironmentHandler handler;
//    private PersonStorage personStorage;
//    private HelicopterStorage helicopterStorage;
//    private TacticalPersonStorage tacticalPersonStorage;
    private CarStorage carStorage;
//    private AirplaneStorage airplaneStorage;
//    private QuadrotorStorage quadrotorStorage;
    private UrbanMap map;
    private WorldInfo worldInfo;

    /**
     * Creates an AgentScoutEnvironment from the provided XML file.
     *
     * @param mapXML the XML file containing the map of the scenario village
     */
    public AgentScoutEnvironment(AgentScoutUniverse universe, File mapXML) {
        super(universe.getSimulation());
        map = new UrbanMap(this, mapXML);
        worldInfo = new WorldInfo();
        this.universe = universe;

        handler = new AgentScoutEnvironmentHandler();

//        personStorage = new PersonStorage();
//        helicopterStorage = new HelicopterStorage();
        carStorage = new CarStorage();
//        tacticalPersonStorage = new TacticalPersonStorage();
//        airplaneStorage = new AirplaneStorage();
//        quadrotorStorage = new QuadrotorStorage();
        worldInfo.initBulletPhysics();
    }

    public UrbanMap getMap() {
        return map;
    }

    @Override
    public AgentScoutEnvironmentHandler handler() {
        return handler;
    }

//    public PersonStorage getPersonStorage() {
//        return personStorage;
//    }
//
//    public HelicopterStorage getHelicopterStorage() {
//        return helicopterStorage;
//    }
    public CarStorage getCarStorage() {
        return carStorage;
    }

    public WorldInfo getWorldInfo() {
        return worldInfo;
    }

//    public TacticalPersonStorage getTacticalPersonStorage() {
//        return tacticalPersonStorage;
//    }
//
//    public AirplaneStorage getAirplaneStorage() {
//        return airplaneStorage;
//    }
//
//    public QuadrotorStorage getQuadrotorStorage() {
//        return quadrotorStorage;
//    }
    public class AgentScoutEnvironmentHandler extends EventBasedHandler {

        public AgentScoutEnvironmentHandler() {
        }

        @Override
        public <C extends Action> C addAction(Class<C> clazz, Entity entity) {
            return instantiateEnvironmentClass(clazz, entity, new Class<?>[]{AgentScoutEnvironment.class, Entity.class});
        }

        @Override
        public <C extends Sensor> C addSensor(Class<C> clazz, Entity entity) {
            return instantiateEnvironmentClass(clazz, entity, new Class<?>[]{AgentScoutEnvironment.class, Entity.class});
        }

        public DefaultCommunicator addDefaultCommunicator(Entity entity) {
            DefaultCommunicator communicator = new DefaultCommunicator(entity.getName());

            try {
                communicator.addChannel(new EventBasedCommunicationChannel(communicator, universe.getSimulation()));
            } catch (CommunicationChannelException e) {
                e.printStackTrace();
            }

            return communicator;
        }

        public DirectoryFacilitator getDirectoryFacilitator() {
            return universe.getDirectoryFacilitator();
        }
//        public void informConvoyAtSafehouse() {
//            universe.createCapturingSubScenario();
//        }
//
//        public String getLeaderAgentName() {
//            return universe.getLeaderAgentName();
//        }

        public UrbanMap getMap() {
            return map;
        }

        public Terrain getTerrain() {
            return worldInfo.getTerrain();
        }

        public CollisionZone getCollisionZone() {
            return worldInfo.getCollisionZone();
        }

        public float getAltitude(float x, float y) {
            return worldInfo.getAltitude(x, y);
        }
    }
}
