package tacv.universe.environment.actuator;

import cz.agents.alite.common.entity.Entity;
import cz.agents.alite.common.event.Event;
import cz.agents.alite.environment.eventbased.EventBasedAction;
import tacv.universe.entity.embodiment.AutomaticRegularCar;
import tacv.universe.environment.AgentScoutEnvironment;
import tacv.universe.environment.storage.CarStorage;
import tacv.util.Point;
import tacv.util.TacEventType;
import tacv.vis.VisualPlanLayer;

public class GoToWaypointCarActuator extends EventBasedAction {

    private static final long STEP_DURATION = 100;
    private static final float WAYPOINT_REACHED_TOLERANCE = 3.5f;
    private CarStorage carStorage;
    private long lastTime = 0;
    private Point targetWaypoint;

    public GoToWaypointCarActuator(AgentScoutEnvironment environment,
            Entity relatedEntity) {
        super(environment, relatedEntity);

        carStorage = environment.getCarStorage();

        getEventProcessor().addEvent(TacEventType.STEP, GoToWaypointCarActuator.this, null, null);
    }

    public void actGoToWaypoint(Point targetWaypoint, Float targetVelocity, boolean isForward) {
        this.targetWaypoint = targetWaypoint;

        // set waypoint to car
        carStorage.setNextWaypoint(getRelatedEntity().getName(), targetWaypoint, targetVelocity, isForward);
    }

    public void setVisualPlan(VisualPlanLayer.VisualPlan visualPlan) {
        carStorage.getCars().get(getRelatedEntity().getName()).setVisualPlan(visualPlan);
    }

    public void actHalt() {
        this.targetWaypoint = null;

        carStorage.setNextWaypoint(getRelatedEntity().getName(), null, null, true);
    }

    @Override
    public void handleEvent(Event event) {
        if (event.isType(TacEventType.STEP)) {
            simpleUpdate(event.getTime());
        } else if (event.isType(TacEventType.WAYPOINT_REACHED)) {
            carStorage.waypointReached(getRelatedEntity().getName());
        } else {
            super.handleEvent(event);
        }
    }

    private void simpleUpdate(long time) {
        AutomaticRegularCar car = carStorage.getCars().get(getRelatedEntity().getName());

        if (lastTime == 0) {
            lastTime = time;
        }
        long duration = time - lastTime;
        lastTime = time;

        carStorage.moveCar(getRelatedEntity().getName(), duration, time);

        // we are at the waypoint
        Double distanceToWaypoint = car.getDistanceToWaypoint();
        if (distanceToWaypoint != null && distanceToWaypoint < WAYPOINT_REACHED_TOLERANCE) {
            // call for new waypoint
            getEventProcessor().addEvent(TacEventType.WAYPOINT_REACHED, this, null, targetWaypoint);
        }

        // drive for ever...
        getEventProcessor().addEvent(TacEventType.STEP, this, null, null, STEP_DURATION);
    }
}
