package tacv.universe.map;

import java.util.ArrayList;
import java.util.List;
import javax.vecmath.Point2d;
import javax.vecmath.Point3d;
import tacv.universe.map.occlusions.BoundingBox;
import tacv.universe.map.occlusions.Wall;
import tacv.util.Point;

/**
 *  A class representing a building at the map. 
 *  Each building is a polyhedron with an quadrilateral base.
 *  
 *  This class is immutable.
 **/

public class Building {
    protected ArrayList<Point> points;
    protected double baseAltitude;
    protected double height;
    
	protected Wall[] walls;
	protected BoundingBox boundingBox;
	
    /**
     * Creates a new building
     * 
     * @param points the 4 points defining the base of the building. 
     * @param baseAltitude the altitude on which base of the building lays
     * @param height the height of the building
     */
    public Building(List<Point> points, double baseAltitude, double height) {
		super();
		this.points = new ArrayList<Point>(points);
		this.baseAltitude = baseAltitude;
		this.height = height;
		
		computeWalls();
	}
    
    private void computeWalls() {
        walls = new Wall[points.size()];
        boundingBox = new BoundingBox();
        
        for (int wallID = 0; wallID < points.size(); wallID++) {
            Point3d point = points.get(wallID);
            //roof[wallID] = new Point3d(point.x, point.y, height);
            boundingBox.addPoint(new Point2d(point.x, point.y));
            
            Wall wall = new Wall();
            wall.id = wallID;
            wall.height = baseAltitude + height;
            wall.point1 = new Point2d(points.get(wallID).x, points.get(wallID).y);
            wall.point2 = new Point2d(points.get((wallID + 1) % points.size()).x, points.get((wallID + 1) % points.size()).y);
            
            wall.point0 = new Point2d((wall.point1.x + wall.point2.x) / 2, (wall.point1.y + wall.point2.y) / 2);
            wall.point1Top = new Point3d(wall.point1.x, wall.point1.y, wall.height);
            wall.point2Top = new Point3d(wall.point2.x, wall.point2.y, wall.height);
            wall.building = this;

            wall.boundingBox = new BoundingBox();
            wall.boundingBox.addPoint(wall.point1);
            wall.boundingBox.addPoint(wall.point2);

            walls[wallID] = wall;
        }

    }

	public ArrayList<Point> getPoints() {
		return new ArrayList<Point>(points);
	}

	public double getBaseAltitude() {
		return baseAltitude;
	}

	public double getHeight() {
		return height;
	}

	public Wall[] getWalls() {
		return walls;
	}
   
}
