package tacv.universe.world;

import cz.agents.alite.planner.spatialmaneuver.zone.GroupZone;
import cz.agents.alite.planner.spatialmaneuver.zone.PrismZone;
import javax.vecmath.Point2d;
import javax.vecmath.Vector2d;
import tacv.universe.world.xml.Box;
import tacv.universe.world.xml.CollisionMesh;

/**
 *
 * @author Jiri Vokrinek
 */
public class CollisionZoneLoader {

    public static GroupZone loadZones(CollisionMesh xmlCollision, double safetyRange) {
        GroupZone collisionZone = new GroupZone();

        for (Box xmlBox : xmlCollision.getBoxes().getBox()) {
            Point2d[] points = new Point2d[4];
            Point2d[] zonePoints = new Point2d[8];
            Point2d[] levelPoints = new Point2d[4];

            points[0] = new Point2d(xmlBox.getPoint().get(0).getX(), xmlBox.getPoint().get(0).getY());
            points[1] = new Point2d(xmlBox.getPoint().get(1).getX(), xmlBox.getPoint().get(1).getY());
            points[2] = new Point2d(xmlBox.getPoint().get(2).getX(), xmlBox.getPoint().get(2).getY());
            points[3] = new Point2d(xmlBox.getPoint().get(3).getX(), xmlBox.getPoint().get(3).getY());
            levelPoints[0] = new Point2d(points[0]);
            levelPoints[1] = new Point2d(points[1]);
            levelPoints[2] = new Point2d(points[2]);
            levelPoints[3] = new Point2d(points[3]);

            Vector2d v = new Vector2d(points[1]);
            v.sub(points[0]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[0] = new Point2d(points[1]);
            zonePoints[0].add(v);

            v = new Vector2d(points[2]);
            v.sub(points[3]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[1] = new Point2d(points[2]);
            zonePoints[1].add(v);

            v = new Vector2d(points[2]);
            v.sub(points[1]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[2] = new Point2d(points[2]);
            zonePoints[2].add(v);

            v = new Vector2d(points[3]);
            v.sub(points[0]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[3] = new Point2d(points[3]);
            zonePoints[3].add(v);

            v = new Vector2d(points[3]);
            v.sub(points[2]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[4] = new Point2d(points[3]);
            zonePoints[4].add(v);

            v = new Vector2d(points[0]);
            v.sub(points[1]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[5] = new Point2d(points[0]);
            zonePoints[5].add(v);

            v = new Vector2d(points[0]);
            v.sub(points[3]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[6] = new Point2d(points[0]);
            zonePoints[6].add(v);

            v = new Vector2d(points[1]);
            v.sub(points[2]);
            v.normalize();
            v.scale(safetyRange);
            zonePoints[7] = new Point2d(points[1]);
            zonePoints[7].add(v);


            collisionZone.add(new PrismZone(zonePoints, 1000));
            //levelZone.add(new PrismZone(levelPoints, 1000));
        }

        // cliffs' zones
        collisionZone.add(new PrismZone(new Point2d[]{
                    new Point2d(308, 393), new Point2d(297, 366), new Point2d(387, 290), new Point2d(413, 332)},
                1000));
//                    levelZone.add(new PrismZone(new Point2d[] {
//                            new Point2d(308, 393 + 30), new Point2d(297 - 30, 366), new Point2d(387, 290 - 30), new Point2d(413 + 30, 332) },
//                            1000));
        collisionZone.add(new PrismZone(new Point2d[]{
                    new Point2d(405, 285), new Point2d(500, 198), new Point2d(539, 205), new Point2d(443, 347)},
                1000));
//                    levelZone.add(new PrismZone(new Point2d[] {
//                            new Point2d(405 - 30, 285), new Point2d(500, 198 - 30), new Point2d(539 + 30, 205), new Point2d(434, 333 + 30) },
//                            1000));
        return collisionZone;
    }
}
