package tacv.vis;

import java.awt.Color;
import java.util.LinkedList;
import java.util.List;

import cz.agents.alite.vis.element.Line;
import cz.agents.alite.vis.element.aggregation.LineElements;
import cz.agents.alite.vis.element.implemetation.LineImpl;
import cz.agents.alite.vis.layer.AbstractLayer;
import cz.agents.alite.vis.layer.VisLayer;
import cz.agents.alite.vis.layer.terminal.LineLayer;
import tacv.universe.map.Building;
import tacv.universe.map.UrbanMap;

public class BuildingLayer extends AbstractLayer {

    BuildingLayer() {}
    
    static VisLayer create(final BuildingsEntity buildingsEntity, final Color color, final int strokeWidth) {
    	LineLayer lineLayer = LineLayer.create(new LineElements() {

            @Override
            public Iterable<Line> getLines() {
                LinkedList<Line> lines = new LinkedList<Line>();
                
                for (Building building : buildingsEntity.getBuildings()) {
                	lines.add(new LineImpl(building.getPoints().get(0), building.getPoints().get(1)));
                	lines.add(new LineImpl(building.getPoints().get(1), building.getPoints().get(2))); 
                	lines.add(new LineImpl(building.getPoints().get(2), building.getPoints().get(3))); 
                	lines.add(new LineImpl(building.getPoints().get(3), building.getPoints().get(0))); 
                }
                
                
                return lines;
            }

            @Override
            public int getStrokeWidth() {
                return strokeWidth;
            }

            @Override
            public Color getColor() {
                return color;
            }

        });
    	
    	lineLayer.setHelpOverrideString("[Building] Layer shows the envelope of the buildings.");
    	
    	return lineLayer;
    	
    }
    

    public static VisLayer create(final UrbanMap mapGraph, final Color color, final int strokeWidth) {
        return create(new BuildingsEntity() {

            @Override
            public List<Building> getBuildings() {
                return mapGraph.getBuildings();
            }

        }, color, strokeWidth);
    }

    private static interface BuildingsEntity {
        public List<Building> getBuildings();
    }

}
