package tacv.vis;

import cz.agents.alite.vis.Vis;
import cz.agents.alite.vis.layer.toggle.ToggleLayer;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;
import tacv.universe.entity.PositionedEntity;
import tacv.universe.environment.storage.PositionedEntityStorage;

public class PositionedEntityToggleLayer extends ToggleLayer {

    private final String entityName;
    private PositionedEntityStorage storage;

    PositionedEntityToggleLayer(String entityName, PositionedEntityStorage storage) {
        this.entityName = entityName;
        this.storage = storage;

        setEnabled(false);
    }

    @Override
    public void init(Vis vis) {
        super.init(vis);

        vis.addMouseListener(new MouseListener() {

            @Override
            public void mouseClicked(MouseEvent e) {
            }

            @Override
            public void mouseEntered(MouseEvent e) {
            }

            @Override
            public void mouseExited(MouseEvent e) {
            }

            @Override
            public void mousePressed(MouseEvent e) {
                if (e.getButton() == MouseEvent.BUTTON1) {
                    if (e.isControlDown()) {
                        setEnabled(true);
                    } else if (storage.getPositionedEntities().containsKey(entityName)) {
                        PositionedEntity entity = storage.getPositionedEntities().get(entityName);
                        double x = entity.getPosition().x;
                        double y = entity.getPosition().y;
                        if (Point2D.distance(Vis.transX(x), Vis.transY(y), e.getX(), e.getY()) < 20) {
                            setEnabled(true);
                        } else if (!e.isShiftDown()) {
                            setEnabled(false);
                        }
                    }
                }
            }

            @Override
            public void mouseReleased(MouseEvent e) {
            }
        });
    }

    @Override
    public String getLayerDescription() {
        String description = "All sub-layers are shown only after left clicking on [" + entityName + "],\n"
                + "by Ctrl+<left click> anywhere, all entities can be selected,\n"
                + "by left clicking anywhere, all entities can be un-selected:";
        return buildLayersDescription(description);
    }

    public static PositionedEntityToggleLayer create(String personName, PositionedEntityStorage personStorage) {
        return new PositionedEntityToggleLayer(personName, personStorage);
    }
}
