package tacv.vis3d.spatial;

import com.jme3.material.Material;
import com.jme3.math.FastMath;
import com.jme3.math.Matrix3f;
import com.jme3.math.Vector3f;
import com.jme3.renderer.queue.RenderQueue.Bucket;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import tacv.util.Point;
import cz.agents.alite.vis3d.Vis3d;
import cz.agents.alite.vis3d.control.AbstractControlExtend;
import cz.agents.alite.vis3d.material.MaterialProvider;
import cz.agents.alite.vis3d.material.MaterialProvider.MaterialType;
import java.util.ArrayList;
import javax.vecmath.Point2d;
import tacv.universe.entity.embodiment.AutomaticRegularCar;
import tacv.vis.VisualPlanLayer.VisualPlan;
import tacv.vis3d.generator.PathVisualization;

public class PathVisualizationControl extends AbstractControlExtend {

    private Vis3d vis;
    private PathVisualization pathVisualization;
    private Spatial carSpatial;
    private Node pathVisualizationNode;
    private AutomaticRegularCar car;
    private int lastVisualPlanHash = 0;

    public PathVisualizationControl(Vis3d vis, PathVisualization pathVisualization, Node pathVisualizationNode, Spatial carSpatial, AutomaticRegularCar car) {
        this.vis = vis;
        this.pathVisualization = pathVisualization;
        this.carSpatial = carSpatial;
        this.pathVisualizationNode = pathVisualizationNode;
        this.car = car;
    }

    @Override
    protected void controlUpdate(float tpf) {
        int visualPlanHash = 0;

        VisualPlan visualPlan = null;
        if (vis.getChasedSpatial() != null) {
            // TODO: refactorize
            if (carSpatial.equals(vis.getChasedSpatial().getParent().getParent())) {
                visualPlan = car.getVisualPlan();
            }
        }
        if (visualPlan != null) {
            visualPlanHash = visualPlan.hashCode();
        }

        if (visualPlanHash != lastVisualPlanHash) {
            lastVisualPlanHash = visualPlanHash;

            pathVisualizationNode.detachAllChildren();

            if (visualPlan != null) {
                ArrayList<Point2d> path = new ArrayList<Point2d>();
                for (Point point : visualPlan) {
                    path.add(new Point2d(point.x, point.y));
                }

                Material mat = MaterialProvider.getInstance(vis).getTransparentMaterial(255, 255, 255, 0.5f, MaterialType.UNSHADED);
                Node buildPathMesh = pathVisualization.buildPathMesh(path, 2, 5, mat);
                buildPathMesh.setMaterial(mat);
                buildPathMesh.setQueueBucket(Bucket.Transparent);

                Matrix3f matrix3f = new Matrix3f();
                matrix3f.fromAngleAxis(-FastMath.HALF_PI, new Vector3f(1, 0, 0));
                buildPathMesh.setLocalRotation(matrix3f);

                pathVisualizationNode.attachChild(buildPathMesh);
            }
        }
    }
}
